(defvar *gray-header-list* nil
  "Contains a list of SMTP header strings to be appended to the standard
header when sending mail to specific addresses

Each entry is of the form:

	(DEST-ADDR . (HEADER . VALUE)")

;; add-to-header-list
(defun add-to-header-list (dest header arg)
  "Add entry to *gray-header-list*

(add-to-header-list DEST-ADDR HEADER ARG)"
  (setq *gray-header-list* (append *gray-header-list*
			      (list (cons dest (cons header arg))))))

;; Produce a list of headers to be added to the message
;; Each list item is a cons of the form:
;;	(HEADER . VALUE)
(defun gray-header-to (dest)
  (setq *gray-header-to-last-input* dest)
  (let ((addr (if (string-match "<\\(.*\\)>" dest)
		  (substring dest (match-beginning 1) (match-end 1))
		dest)))
    (mapcar (function (lambda (x)
			(cond
			 ((string-equal addr (car x))
			  (cdr x)))))
	    *gray-header-list*)))

;; Modify message headers according to destination address
;; NOTE: "Fcc: sent-messages" gets appended anyway.
(defun gray-mh-send-hook ()
  (save-excursion
    (goto-char (point-min))
    (let ((add-header (list (cons "Fcc" "sent-messages"))))
      (while (and (not (eobp))
		  (looking-at "\\(.*\\): *\\(.*\\)"))
	(let* ((kw (buffer-substring (match-beginning 1)
				     (match-end 1)))
	       (arg (buffer-substring (match-beginning 2)
				      (match-end 2)))
	       (fun (intern (concat "gray-header-" (downcase kw)))))
	  (if (fboundp fun)
	      (setq add-header (append add-header (funcall fun arg)))))
	(forward-line 1)
	(while (looking-at "[ \t]") (forward-line 1)))
      (setq *add-header* add-header)
      (mapcar (function (lambda (x)
			  (if (not (null x))
			      (insert (concat (car x) ": " (cdr x) "\n")))))
	      add-header))))

(add-hook 'mh-before-send-letter-hook 'gray-mh-send-hook)
